import paramiko
import socket
import logging
import time
import os
import warnings


class SSH():
    def __init__(self, host, port, username, pwd):
        self.host = host
        self.port = port
        self.username = username
        self.pwd = pwd
        self.transport = None
        self.chan = None

    def status(self):
        if self.transport is None:
            return False
        return self.transport.is_active()

    def connect(self, times=0):
        times += 1
        try:
            logging.info('%s:start connect server...' % time.strftime('%H:%M:%S'))
            socket_ = socket.socket()
            socket_.connect((self.host, self.port))
            transport = paramiko.Transport(socket_)
            transport.connect(username=self.username, password=self.pwd)
            chan = transport.open_session()
            chan.get_pty()
            chan.invoke_shell()
            self.transport = transport
            self.chan = chan
            time.sleep(1)
            logging.info('%s:connect success!' % time.strftime('%H:%M:%S'))
            rec = self.chan.recv(-1)
            content = str(rec, encoding='utf-8')
            # logging.info(content)
            return True
        except Exception as e:
            logging.error('%s' % e)
            if times >= 3:
                logging.error('失败次数已超过3次，不再尝试连接')
                return False
            else:
                logging.error('%s:连接失败，重新连接……' % time.strftime('%H:%M:%S'))
                self.connect(times)

    def close(self):
        if self.chan is not None:
            self.chan.close()
        if self.transport is not None:
            self.transport.close()

    def exec_command(self, cmd, callback=None):
        '''
        执行命令
        :param cmd:
        :param callback:
        :return:
        '''
        ssh_client = paramiko.SSHClient()
        ssh_client._transport = self.transport
        stdin, stdout, stderr = ssh_client.exec_command(cmd + '\n')
        while not stdout.channel.exit_status_ready():
            line = stdout.readline()
            if len(line) != 0:
                print(line, end='')
                logging.info(line[0:-1])
                if callback is not None:
                    callback(line[0:-1])

            # 由于在退出时，stdout还是会有一次输出，因此需要单独处理，处理完之后，就可以跳出了
            if stdout.channel.exit_status_ready():
                lines = stdout.readlines()
                for line in lines:
                    print(line, end='')
                    logging.info(line[0:-1])
                    if callback is not None:
                        callback(line[0:-1])
                break

    def exec_cmd(self, cmd):
        '''
        执行命令，已废弃
        :param cmd:
        :return:
        '''
        warnings.warn("this function is deprecated", DeprecationWarning)
        self.chan.send(cmd + '\n')
        content = ''
        tmp = None
        rec_info = ''
        while True:
            rec = None
            b = self.chan.recv(-1)
            try:
                if tmp is None:
                    rec = str(b, encoding='utf-8')
                else:
                    rec = str(tmp + b, encoding='utf-8')
            except:
                if tmp is None:
                    tmp = b
                else:
                    tmp = tmp + b
                continue
            tmp = None
            print(len(rec), '====================')
            if len(rec) > 0:
                for r in rec:
                    rec_info = '%s%s' % (rec_info, r)
                    if r == '\r':
                        print(rec_info, end='')
                        rec_info = ''
                content = '%s%s' % (content, rec)
            if rec[len(rec) - 1:len(rec)] == '$':
                break
        print(content, end='')
        logging.info(content, False)
        return content

    def download(self, src_files, dst_dir, file_rename=None):
        '''
        下载src_files列表到本地dst_dir目录，可根据需要重命名file_rename
        :param src_files:
        :param dst_dir:
        :param file_rename:
        :return:
        '''
        if not os.path.exists(dst_dir):
            os.makedirs(dst_dir)
        result = 0
        if file_rename is not None and len(file_rename) != len(src_files):
            return -1
        file_names = []
        sftp = paramiko.SFTPClient.from_transport(self.transport)
        for i in range(len(src_files)):
            src_file = src_files[i]
            try:
                sftp.stat(src_file)
            except Exception as e:
                logging.error('%s:下载文件%s不存在！' % (time.strftime('%H:%M:%S'), src_file))
                result = -1
                continue
            index = src_file.rfind('/')
            file_name = src_file[index + 1:len(src_file)]
            if file_rename is not None:
                file_name = file_rename[i]
            dst_path = os.path.join(dst_dir, file_name)
            sftp.get(src_file, dst_path)
            file_names.append(file_name)
            logging.info('下载文件成功,保存路径：【%s】' % dst_path)
        sftp.close()
        return result

    def download_all(self, src_dir, dst_dir, filter_func=None, callback=None):
        '''
        下载远程目录src_dir中所有文件到本地dst_dir目录，可用filter_func对文件进行过滤
        :param src_dir:
        :param dst_dir:
        :param filter_func:
        :param callback:
        :return:
        '''
        if not os.path.exists(dst_dir):
            os.makedirs(dst_dir)
        sftp = paramiko.SFTPClient.from_transport(self.transport)
        files = sftp.listdir(src_dir)
        if filter_func is not None:
            files = filter_func(files)
        for file in files:
            src_file = src_dir + '/' + file
            local_file = dst_dir + '/' + file
            sftp.get(src_file, local_file)
            logging.info(f'download file from 【{src_file}】 to 【{local_file}】')
            if callback is not None:
                callback(f'download file from 【{src_file}】 to 【{local_file}】')
        sftp.close()

    def upload(self, src_files, dst_dir, file_rename=None, callback=None):
        '''
        上传本地文件src_files到远程目录dst_dir
        :param src_files:
        :param dst_dir:
        :param file_rename:
        :param callback:
        :return:
        '''
        try:
            result = 0
            if file_rename is not None and len(file_rename) != len(src_files):
                return -1
            sftp = paramiko.SFTPClient.from_transport(self.transport)
            for i in range(len(src_files)):
                src_file = src_files[i]
                if not os.path.exists(src_file):
                    logging.error('未找到要上传的文件【%s】' % src_file)
                    if callback is not None:
                        callback('未找到要上传的文件【%s】' % src_file)
                    continue

                index = src_file.rfind('/')
                file_name = src_file[index + 1:len(src_file)]
                if file_rename is not None:
                    file_name = file_rename[i]
                dst_path = dst_dir + '/' + file_name
                # dst_path = os.path.join(dst_dir, file_name)
                print(src_file, dst_path)
                sftp.put(src_file, dst_path)
                logging.info('上传文件成功,保存路径：【%s】' % dst_path)
                if callback is not None:
                    callback('上传文件成功,保存路径：【%s】' % dst_path)
            sftp.close()
            return result
        except Exception as e:
            print(e)
            exit()

    def upload_dir(self, local_dir, remote_dir, callback=None):
        '''
        上传本地文件夹到远程目录
        :param local_dir:
        :param remote_dir:
        :param callback:
        :return:
        '''
        local_dir = local_dir.replace('\\', '/').strip()
        if local_dir[-1] == '/':
            local_dir = local_dir[0:-1]
        dir_name = local_dir[local_dir.rindex('/') + 1:].replace('/', '')
        sftp = paramiko.SFTPClient.from_transport(self.transport)
        for parent, dirnames, filenames in os.walk(local_dir):
            for filename in filenames:
                parent = parent.replace('\\', '/')
                local_file = os.path.join(parent, filename).replace('\\', '/')
                remote_file = local_file.replace(local_dir, remote_dir + '/' + dir_name)

                remote_file_dir = remote_file[0:remote_file.rindex('/')]
                try:
                    sftp.stat(remote_file_dir)
                except Exception as e:
                    cmd = 'mkdir -p %s' % remote_file_dir
                    print(cmd)
                    if callback is not None:
                        callback(cmd)
                    self.exec_command(cmd)

                sftp.put(local_file, remote_file)
                print(f'upload file from {local_file} to {remote_file}')
                if callback is not None:
                    callback(f'upload file from {local_file} to {remote_file}')

    def check_file(self, src_file, stop_time=None, cur_date=None):
        '''
        个性化需求（校验文件是否在9:25之后更新）
        :param src_file:
        :param stop_time:
        :param cur_date:
        :return:
        '''
        check = False
        sftp = paramiko.SFTPClient.from_transport(self.transport)
        while 1:
            try:
                if stop_time is not None:
                    current_time = int(time.strftime('%H%M%S'))
                    if current_time >= int(stop_time.replace(':', '')):
                        break
                attr = sftp.stat(src_file)
                if cur_date is not None:
                    mdate_time = int(TimeStampToTime(attr.st_mtime).replace('-', '').replace(':', '').replace(' ', ''))
                    if mdate_time >= int(cur_date) * 1000000 + 92500:
                        check = True
                        break
                else:
                    check = True
                    break
                time.sleep(1)
            except:
                time.sleep(1)
        sftp.close()
        return check

    def is_file_update(self, files, check_time):
        '''
        个性化需求（校验文件是否在check_time之后更新）
        :param files:
        :param check_time:
        :return:
        '''
        check = False
        sftp = paramiko.SFTPClient.from_transport(self.transport)
        update_info_dict = {}
        for file in files:
            update_info_dict[file] = [0, int(check_time), 0]  # 文件大小、时间、是否更新完成
        update_count = 0
        while 1:
            print(time.strftime('%H:%M:%S'), 'checking update')
            try:
                for file in files:
                    attr = sftp.stat(file)
                    mdate_time = int(TimeStampToTime(attr.st_mtime).replace('-', '').replace(':', '').replace(' ', ''))
                    file_size = attr.st_size
                    if mdate_time < int(check_time):
                        continue
                    else:
                        last_file_size = update_info_dict[file][0]
                        last_update_time = update_info_dict[file][1]
                        if file_size != last_file_size or mdate_time != last_update_time:
                            update_info_dict[file][0] = file_size
                            update_info_dict[file][1] = mdate_time
                            continue
                        else:
                            update_info_dict[file][2] = 1
                print(update_info_dict)
                for file in files:
                    update_count += update_info_dict[file][2]
                if update_count == len(files):
                    break
                time.sleep(1)
            except:
                time.sleep(1)
        sftp.close()
        return check


def TimeStampToTime(timestamp):
    timeStruct = time.localtime(timestamp)
    return time.strftime('%Y-%m-%d %H:%M:%S', timeStruct)



import pandas as pd
import pymysql
from pymysql.cursors import DictCursor

conn_pool = {}


def add_connect(host, port, user, passwd, db, name):
    global conn_pool
    db = DBConn(host, port, user, passwd, db, name)
    conn_pool[db.name] = db


def get_conn(name):
    global conn_pool
    current_conn = conn_pool[name]
    return current_conn


class DBConn():
    def __init__(self, host, port, user, passwd, db, name):
        self.name = name
        self.conn = pymysql.connect(user=user, passwd=passwd, host=host, port=port, db=db)
        self.cur = self.conn.cursor(DictCursor)

    def close(self):
        if self.conn is not None:
            self.conn.close()

    def query_data(self, sql, args=None):
        self.cur.execute(sql, args)
        results = self.cur.fetchall()
        df = pd.DataFrame(results)
        return df

    def exec_sql(self, sql, args=None):
        self.cur.execute(sql, args)
        self.conn.commit()

    def exec_many(self, sql, args=None):
        self.cur.executemany(sql, args)
        self.conn.commit()



from pandas import DataFrame
import logging
import os
import numpy as np
import stat, errno, shutil
import random
import string
from datetime import datetime
import time


def code_process(codes, add_market=False):
    '''
    处理股票代码
    :param codes:
    :param add_market:
    :return:
    '''

    def transform(code, add_market):
        if isinstance(code, int) or isinstance(code, float):
            code = str(int(code))
        code = code.zfill(6)
        if add_market:
            if code[0:1] == '6':
                code += '.SH'
            elif code[0:1] == '0' or code[0:1] == '3':
                code += '.SZ'
        return code

    if isinstance(codes, int) or isinstance(codes, float) or isinstance(codes, str):
        return transform(codes, add_market)
    elif isinstance(codes, list):
        return [transform(x, add_market) for x in codes]
    elif isinstance(codes, DataFrame):
        codes['code'] = codes['code'].apply(lambda x: transform(x, add_market))
        return codes


def init(log_file):
    '''
    初始化日志
    :param log_file:
    :return:
    '''
    LOG_FORMAT = "%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s - %(message)s"  # 日志格式化输出
    DATE_FORMAT = "%Y/%m/%d %H:%M:%S %p"  # 日期格式
    if not os.path.exists('log'):
        os.mkdir('./log')
    fp = logging.FileHandler('log/%s' % log_file, encoding='utf-8')
    fs = logging.StreamHandler()
    params = {'level': logging.INFO, 'format': LOG_FORMAT, 'datefmt': DATE_FORMAT, 'handlers': [fp, fs]}
    logging.basicConfig(**params)  # 调用


def get_trade_days():
    '''
    获取交易日列表，trade_dates.txt会不定期更新
    :return:
    '''
    file_dir = os.path.dirname(os.path.abspath(__file__))
    data = np.loadtxt(os.path.join(file_dir, 'trade_dates.txt'), dtype=str).tolist()
    return data


def is_trade_day(date):
    '''
    判断是否交易日
    :param date:
    :return:
    '''
    trade_dates = get_trade_days()
    date = str(date).replace('/', '').replace('-', '')
    return date in trade_dates


def get_trade_day(date, offset):
    '''
    交易日前推或者后推
    :param date:
    :param offset:
    :return:
    '''
    trade_dates = get_trade_days()
    date = str(date).replace('/', '').replace('-', '')
    if date not in trade_dates:
        return -1
    index = trade_dates.index(date)
    if index + offset < 0 or index + offset >= len(trade_dates):
        return None
    return trade_dates[index + offset]


def rm_dir(path):
    def handle_remove_read_only(func, path, exc):
        excvalue = exc[1]
        if func in (os.rmdir, os.remove, os.unlink) and excvalue.errno == errno.EACCES:
            os.chmod(path, stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)  # 0777
            func(path)
        else:
            raise

    shutil.rmtree(path, onerror=handle_remove_read_only)


def random_id(n=12):
    '''
    生成一个n位的随机ID
    :param n:
    :return:
    '''
    ran_str = ''.join(random.sample('ABCDEF' + string.digits, n))
    return ran_str


def file_block_check(record_file, pre_time_point):
    current_datetime = datetime.now()
    while True:
        with open(record_file, 'r') as file:
            content = file.read()
            update_time = datetime.strptime(content, '%Y-%m-%d %H:%M:%S')
            print(f'当前时间：{current_datetime}，文件更新时间：{update_time},预测时间节点：{pre_time_point}')
            if current_datetime > update_time > pre_time_point:
                print(f'当前时间大于文件更新时间大于预测时间节点，开始预测')
                break
        time.sleep(1)
        current_datetime = datetime.now()


def find_path(file, dir):
    try:
        fs = os.scandir(dir)
        for s in fs:
            # path = os.path.join(dir, s)
            if s.is_dir():
                result = find_path(file, s.path)
                if result is None:
                    continue
                else:
                    return result
            else:
                if s.name == file:
                    return s.path
        return None
    except:
        print(dir + "  目录无访问权限")


def pytree(dir, level=0):
    def create_dir_content(dir_name, level, is_last):
        if level == 0:
            return dir_name
        elif level == 1:
            if not is_last:
                return f'├── {dir_name}'
            else:
                return f'└── {dir_name}'
        else:
            pass

    base_name = os.path.basename(dir)
    if level == 0:
        print(base_name)
    else:
        print('├' + ' ' * 3 * (level - 1) + '──' + base_name)
    try:
        fs = os.scandir(dir)
        for s in fs:
            if s.is_dir():
                pytree(s.path, level + 1)
            # else:
            #     print('├' * level + '──' + s.name)
        return None
    except:
        pass


def tree(path, depth=0):
    if depth == 0:
        print(path)
    items = os.listdir(path)
    for i in range(len(items)):
        item = items[i]
        # 输出文件夹中的文件和子文件夹名
        if i != len(items) - 1:
            print('│    ' * depth, end='')
            print('├──', item)
        else:
            print('     ' * depth, end='')
            print('└──', item)
        item = os.path.join(path, item)
        if os.path.isdir(item):
            tree(item, depth + 1)


if __name__ == '__main__':
    tree('D:/test')



import yagmail


def send_mail(user, password, host, port, subject, to, contents=None, cc=None, attachments=None):
    yag = yagmail.SMTP(user=user, password=password, host=host, port=port)
    yag.send(
        to=to,  # 如果多个收件人的话，写成list就行了，如果只是一个账号，就直接写字符串就行to='123@qq.com'
        cc=cc,  # 抄送
        subject=subject,  # 邮件标题
        contents=contents,  # 邮件正文
        attachments=attachments)  # 附件如果只有一个的话，用字符串就行，attachments=r'd://baidu_img.jpg'
    yag.close()



import requests


def wx_send(token, text, desp):
    '''
    发送信息到微信(虾推啥)
    :param text:
    :param desp: 支持html格式
    :return:
    '''
    mydata = {
        'text': text,
        'desp': desp
    }

    resp = requests.post('http://wx.xtuis.cn/%s.send' % token, data=mydata)
    return resp



from setuptools import setup

setup(name='personal_lib',
      version='1.0.10',
      description="a personal library",
      long_description = "a personal library",
      packages=['personal_lib', 'personal_lib/utils'],
      package_data={'': ['*.txt']},
      install_requires=['pandas>=1.1.4', 'yagmail>=0.14.261', 'paramiko>=2.7.2', 'pymysql>=1.0.2'],
      python_requires='>=3.7',
      )
